// coding.c
//
//  Functions for converting between the sprite palette bitmaps
//  and the format of the memory rom images.
//
//  November, 1998
//
//   Ivan Mackintosh

#include <stdio.h>
#include "allegro.h"
#include "sprtplte.h"
#include "general.h"
#include "gamedesc.h"
#include "coding.h"
#include "toolmenu.h"



int GetBit(long startbit)
{
   long nbyte = startbit/8;
   int nbit = startbit%8;
   int mask = 0x80 >> nbit;

   return ((GfxRomData[nbyte] & mask) == mask);
}


void Decode(int CurrentBank)
{
   GFXBANKEXTRA * gbe = &GfxBankExtraInfo[CurrentBank];
   SPRITE_PALETTE * sp = &GfxBanks[CurrentBank];
	int x, y, spriteno, plane;
	BYTE result = 0;
	long StartBit = gbe->startaddress * 8;
   BITMAP * tbmp = create_bitmap(sp->sprite_w,sp->sprite_h);

   clear_to_color(tbmp, FIRST_USER_COLOR);

   for (spriteno = 0; spriteno < sp->n_total; spriteno++)
	{
      // copy the current character into a bitmap
      for (y = 0; y < sp->sprite_h; y++)
      {
         for (x = 0; x < sp->sprite_w; x++)
         {
            result = 0;
            for (plane = 0; plane < gbe->planes; plane++)
            {
               if (GetBit(StartBit +
                          gbe->xoffset[x] +
                          gbe->yoffset[y] +
                          gbe->charincrement * spriteno +
                          gbe->planeoffsets[plane]))
                  result ^= (1 << (gbe->planes - 1 - plane));
            }
            putpixel(tbmp, x, y, result + FIRST_USER_COLOR);
         }
      }

      // now change its orientation
      switch(Orientation)
      {
         case ORIENTATION_FLIP_X:
            tools_flip(tbmp, TOOLS_HORIZ_FLIP);
            break;
         case ORIENTATION_FLIP_Y:
            tools_flip(tbmp, TOOLS_VERT_FLIP);
            break;
         case ORIENTATION_ROTATE_180:
         case ORIENTATION_SWAP_XY:
            tools_cw_rotate(tbmp);
            tools_cw_rotate(tbmp);
            break;
         case ORIENTATION_ROTATE_90:
            tools_cw_rotate(tbmp);
            break;
         case ORIENTATION_ROTATE_270:
            tools_cw_rotate(tbmp);
            tools_cw_rotate(tbmp);
            tools_cw_rotate(tbmp);
            break;
         default:
            break;

      }

      // now copy it to the appropriate place in the master gfxbank bitmap
      blit(tbmp, sp->bmp, 0, 0, spriteno * sp->sprite_w, 0, sp->sprite_w, sp->sprite_h);
	}

   destroy_bitmap(tbmp);
}


void PutBit(long startbit, int value)
{
	long nbyte = startbit/8;
	int nbit = startbit%8;
	int mask = 0x80 >> nbit;

	GfxRomData[nbyte] &= ~mask;

	if (value != 0)
		GfxRomData[nbyte] |= mask;
}



void Encode(int CurrentBank)
{
   GFXBANKEXTRA * gbe = &GfxBankExtraInfo[CurrentBank];
   SPRITE_PALETTE * sp = &GfxBanks[CurrentBank];
	int x, y, spriteno, plane;
	long StartBit = gbe->startaddress * 8;
   BITMAP * tbmp = create_bitmap(sp->sprite_w,sp->sprite_h);

   for (spriteno = 0; spriteno < sp->n_total; spriteno++)
	{
      // copy the current char into the temp bitmap for rotation
      blit(sp->bmp, tbmp, spriteno * sp->sprite_w, 0, 0, 0, sp->sprite_w, sp->sprite_h);

      // do any rotation here.
      // switch the orientation back to the correct one
      // note we must do the opposite to decode here (this only affects
      // rotate 90 and 270 though)
      switch(Orientation)
      {
         case ORIENTATION_FLIP_X:
            tools_flip(tbmp, TOOLS_HORIZ_FLIP);
            break;
         case ORIENTATION_FLIP_Y:
            tools_flip(tbmp, TOOLS_VERT_FLIP);
            break;
         case ORIENTATION_ROTATE_180:
         case ORIENTATION_SWAP_XY:
            tools_cw_rotate(tbmp);
            tools_cw_rotate(tbmp);
            break;
         case ORIENTATION_ROTATE_90:
            // rotate 270 to get back to original orientation
            tools_cw_rotate(tbmp);
            tools_cw_rotate(tbmp);
            tools_cw_rotate(tbmp);
            break;
         case ORIENTATION_ROTATE_270:
            // rotate 90 to get back to original orientation
            tools_cw_rotate(tbmp);
            break;
         default:
            break;
      }

      // now copy the bitmap into the memory rom image in the appropriate format
      for (y = 0; y < sp->sprite_h; y++)
      {
         for (x = 0; x < sp->sprite_w; x++)
         {
            for (plane = 0; plane < gbe->planes; plane++)
            {
               PutBit(StartBit +
                      gbe->xoffset[x] +
                      gbe->yoffset[y] +
                      gbe->charincrement * spriteno +
                      gbe->planeoffsets[plane],
                      (getpixel(tbmp, x, y) - FIRST_USER_COLOR)
                         & (1 << (gbe->planes - 1 - plane)));
            }
         }
      }
   }
               
   destroy_bitmap(tbmp);
}




// If and old bank is specified (-1 for no old bank) then
// the bitmap for that bank is encoded into the memory roms image
// Once this is complete the bitmap for the new bank is then
// decode from the memory rom image.
void SwitchGraphicsBank(int oldbank, int newbank)
{
   // anything to encode?
   if (oldbank != -1)
   {
      Encode(oldbank);
   }

   // now decode the new one
   Decode(newbank);
}
